/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_ACCEPTOR_WORKING_NB_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_ACCEPTOR_WORKING_NB_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS;

import java.io.ByteArrayOutputStream;
import java.util.List;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.base.junit.shell.SimpleShell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.mock.PetalsJmxApiFactoryMock;
import org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock;

/**
 * Unit tests of {@link MexAcceptors}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class MexAcceptorsTest extends AbstractTestCase {

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new MexAcceptors();
        try {
            monitoringSubFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new MexAcceptors();
        try {
            monitoringSubFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided does not exist.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException, ConnectionErrorException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        final MonitoringSubFunction monitoringSubFunction = new MexAcceptors();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] { "-" + MexAcceptors.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>error occurs getting monitoring values.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_01() throws MonitoringSubFunctionException, ConnectionErrorException {

        final MonitoringSubFunction monitoringSubFunction = new MexAcceptors();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] { "-" + MexAcceptors.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max number of working message exchange acceptors,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max number of working message exchange acceptors is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexAcceptors();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        MEX_ACCEPTOR_WORKING_NB_CURRENT = Long.valueOf(123);
        MEX_ACCEPTOR_WORKING_NB_MAX = null;
        MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS = new Long[] { Long.valueOf(1000), Long.valueOf(500),
                Long.valueOf(100), Long.valueOf(900), Long.valueOf(400), Long.valueOf(150) };
        monitoringSubFunction.execute(new String[] { "-" + MexAcceptors.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("the metric value associated to the max working message exchange acceptors is displayed",
                output.contains(MexAcceptors.HEADER_ACCEPTOR_WORKING_NB_MAX));
        assertTrue("the metric value associated to the current number of message exchange acceptors is not displayed",
                output.contains(MexAcceptors.HEADER_ACCEPTOR_WORKING_NB_CURRENT));
        assertTrue(
                "the metric value associated to the duration of working message exchange acceptors is not displayed",
                output.contains(MexAcceptors.HEADER_ACCEPTOR_WORKING_NB_CURRENT));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current number of working message exchange
     * acceptors</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current number of working message exchange acceptors is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_03() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexAcceptors();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        MEX_ACCEPTOR_WORKING_NB_CURRENT = null;
        MEX_ACCEPTOR_WORKING_NB_MAX = Long.valueOf(123);
        MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS = new Long[] { Long.valueOf(1000), Long.valueOf(500),
                Long.valueOf(100), Long.valueOf(900), Long.valueOf(400), Long.valueOf(150) };
        monitoringSubFunction.execute(new String[] { "-" + MexAcceptors.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });
        monitoringSubFunction.execute(new String[] { "-" + MexAcceptors.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the max working message exchange acceptors is not displayed",
                output.contains(MexAcceptors.HEADER_ACCEPTOR_WORKING_NB_MAX));
        assertFalse("the metric value associated to the current number of message exchange acceptors is displayed",
                output.contains(MexAcceptors.HEADER_ACCEPTOR_WORKING_NB_CURRENT));
        assertTrue(
                "the metric value associated to the duration of working message exchange acceptors is not displayed",
                output.contains(MexAcceptors.HEADER_ACCEPTOR_DURATION_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>no null value is returned for metric,</li>
     * <li>no error will occur.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexAcceptors();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        MEX_ACCEPTOR_WORKING_NB_CURRENT = Long.valueOf(791);
        MEX_ACCEPTOR_WORKING_NB_MAX = Long.valueOf(123);
        MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS = new Long[] { Long.valueOf(1000), Long.valueOf(500),
                Long.valueOf(100), Long.valueOf(900), Long.valueOf(400), Long.valueOf(150) };
        monitoringSubFunction.execute(new String[] { "-" + MexAcceptors.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        boolean isMissingMexAcceptorNbMax = true;
        boolean isMissingMexAcceptorNbCurrent = true;
        boolean isMissingMexAcceptorDurationMax = true;
        boolean isMissingMexAcceptorDurationAvg = true;
        boolean isMissingMexAcceptorDurationMin = true;
        boolean isMissingMexAcceptorDuration10P = true;
        boolean isMissingMexAcceptorDuration50P = true;
        boolean isMissingMexAcceptorDuration90P = true;
        final StringTokenizer strTokenizer = new StringTokenizer(output);
        while (strTokenizer.hasMoreElements()) {
            final StringTokenizer metricTokenizer = new StringTokenizer(strTokenizer.nextToken(), ":");
            final String metricLabel = metricTokenizer.nextToken();
            assertNotNull("missing metric label", metricLabel);
            final String metricValue = metricTokenizer.nextToken();
            assertNotNull("missing metric value", metricValue);
            if (MexAcceptors.HEADER_ACCEPTOR_WORKING_NB_CURRENT.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current number of working message exchange acceptors",
                        MEX_ACCEPTOR_WORKING_NB_CURRENT, Long.valueOf(metricValue));
                isMissingMexAcceptorNbCurrent = false;
            } else if (MexAcceptors.HEADER_ACCEPTOR_WORKING_NB_MAX.equals(metricLabel)) {
                assertEquals("invalid metric value associated to the max number of message exchange acceptors",
                        MEX_ACCEPTOR_WORKING_NB_MAX, Long.valueOf(metricValue));
                isMissingMexAcceptorNbMax = false;
            } else if (MexAcceptors.HEADER_ACCEPTOR_DURATION_MAX.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max duration of working message exchange acceptors",
                        MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS[0], Long.valueOf(metricValue));
                isMissingMexAcceptorDurationMax = false;
            } else if (MexAcceptors.HEADER_ACCEPTOR_DURATION_AVG.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the avg duration of working message exchange acceptors",
                        MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS[1], Long.valueOf(metricValue));
                isMissingMexAcceptorDurationAvg = false;
            } else if (MexAcceptors.HEADER_ACCEPTOR_DURATION_MIN.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the min duration of working message exchange acceptors",
                        MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS[2], Long.valueOf(metricValue));
                isMissingMexAcceptorDurationMin = false;
            } else if (MexAcceptors.HEADER_ACCEPTOR_DURATION_10P.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the 10-percentile duration of working message exchange acceptors",
                        MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS[3], Long.valueOf(metricValue));
                isMissingMexAcceptorDuration10P = false;
            } else if (MexAcceptors.HEADER_ACCEPTOR_DURATION_50P.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the 50-percentile duration of working message exchange acceptors",
                        MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS[4], Long.valueOf(metricValue));
                isMissingMexAcceptorDuration50P = false;
            } else if (MexAcceptors.HEADER_ACCEPTOR_DURATION_90P.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the 90-percentile duration of working message exchange acceptors",
                        MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS[5], Long.valueOf(metricValue));
                isMissingMexAcceptorDuration90P = false;
            } else {
                fail("Unexpected metric");
            }
        }
        assertFalse(
                "The metric value associated to the current number of working message exchange acceptor is missing",
                isMissingMexAcceptorNbCurrent);
        assertFalse("The metric value associated to the max number of working message exchange acceptors",
                isMissingMexAcceptorNbMax);
        assertFalse("The metric value associated to the max duration of working message exchange acceptors",
                isMissingMexAcceptorDurationMax);
        assertFalse("The metric value associated to the avg duration of working message exchange acceptors",
                isMissingMexAcceptorDurationAvg);
        assertFalse("The metric value associated to the min duration of working message exchange acceptors",
                isMissingMexAcceptorDurationMin);
        assertFalse("The metric value associated to the 10-percentile duration of working message exchange acceptors",
                isMissingMexAcceptorDuration10P);
        assertFalse("The metric value associated to the 50-percentile duration of working message exchange acceptors",
                isMissingMexAcceptorDuration50P);
        assertFalse("The metric value associated to the 90-percentile duration of working message exchange acceptors",
                isMissingMexAcceptorDuration90P);
    }
}
